# -*- coding: utf-8 -*-
import sys
import json
import numpy as np
import netCDF4
import xarray as xr
import matplotlib as mpl
import matplotlib.pyplot as plt
import datetime

########
#ro = 1036.0
ro_scl = 1024.5

if (len(sys.argv) < 2):
    print ('Usage: ' + sys.argv[0] + ' start_year end_year longitude')
    sys.exit()

styr = int(sys.argv[1])
edyr = int(sys.argv[2])
lon_draw = float(sys.argv[3])
nyr = edyr-styr+1

title = [ '(a) GLB100km', '(b) NP10km',
          '(c) NP10km - GLB100km', '(d) GLODAPv2 and WOA13v2' ]

xlim = [ [-20, 65], [-20, 65], [-20, 65], [-20, 65] ]


####################################################################
path_glb100km='../data/glb100km/'
path_np10km='../data/np10km/'
path_ref='../data/refdata/'
####################################################################
# Ref data

print( "Loading GLODAPv2 data" )

path_glodap= path_ref + 'GLODAPv2/'
reffile = path_glodap + 'GLODAPv2.2016b.Cant.nc'
ncref = netCDF4.Dataset(reffile,'r')
lon_glo = ncref.variables['lon'][:]
lat_glo = ncref.variables['lat'][:]
dep_glo = ncref.variables['Depth'][:]
nx = len(ncref.dimensions['lon'])
ny = len(ncref.dimensions['lat'])
nz = len(ncref.dimensions['depth_surface'])
cant_3d = ncref.variables['Cant'][:,:,:]
ncref.close()

cant_3d = np.where(cant_3d < -990, np.NaN, cant_3d * ro_scl * 1.e-3) # mmol / m3
print("GLODAPv2 depth ", dep_glo[0:nz])

DS_ref = xr.Dataset( {'cant': (['depth','lat','lon'], cant_3d),},
                      coords = { 'depth': dep_glo, 'lat': lat_glo,'lon': lon_glo, } )
DS_ref = DS_ref.roll(lon=20, roll_coords=True)

#print(DS_ref)

print( "Loading WOA13v2 data" )

path_woa_gr = path_ref + 'WOA13v2/1deg_L102/'
path_woa_nc = path_ref + 'WOA13v2/1deg_L102/'
tmpfile = path_woa_nc + 'woa13_decav_t00_01v2.nc'
ncref = netCDF4.Dataset(tmpfile,'r')
#lon_woa = ncref.variables['lon'][:]
#lat_woa = ncref.variables['lat'][:]
dep_woa = ncref.variables['depth'][:]
nx_woa = len(ncref.dimensions['lon'])
ny_woa = len(ncref.dimensions['lat'])
nz_woa = len(ncref.dimensions['depth'])
ncref.close()

print("WOA13v2 depth ", dep_woa[0:nz_woa])

sig0 = np.array(np.empty((nz_woa,ny_woa,nx_woa)),dtype=np.float64)
sig0_file = path_woa_gr + 'woa13_decav_sigma000.gd'
fl = open(sig0_file,'rb')
for k in range(nz_woa):
    for j in range(ny_woa):
        sig0[k,j,:] = np.fromfile(fl, dtype = '>f', count= nx_woa)

sig0 = np.where(sig0 < -9.0e33, np.NaN, sig0)

DS_sig0_ref = xr.Dataset( {'sigma0': (['depth','lat','lon'], sig0),},
                          coords = { 'depth': dep_woa,
                                     'lat': np.linspace(-89.5,89.5,num=180),
                                     'lon': np.linspace(0.5,359.5,num=360) } )
print(DS_sig0_ref)

####################################################################

time = np.empty(edyr-styr+1,dtype='object')
for yr in range(styr,edyr+1):
    time[yr-styr] = datetime.datetime(yr,1,1)

####################################################################
# GLB100km

print( "Loading GLB100km data" )

path_g100 = path_glb100km + 'run-20211104o2/ANALY/annual'

base_dic_g100 = 'hs_DIC_woa1x1'
base_dic_prei_g100 = 'hs_DIC_prei_woa1x1'

suffix = '.' + '{:04}'.format(yr)
dic_file = path_g100 + '/' + base_dic_g100 + suffix
ncg100 = netCDF4.Dataset(dic_file,'r')
lon_mod = ncg100.variables['lon'][:]
lat_mod = ncg100.variables['lat'][:]
dep_mod = ncg100.variables['depth'][:]
nx = len(ncg100.dimensions['lon'])
ny = len(ncg100.dimensions['lat'])
nz = len(ncg100.dimensions['depth'])
ncg100.close()

ix=0
londist=abs(lon_mod[ix]-lon_draw)
for i in range(1,nx):
   if (abs(lon_mod[i]-lon_draw) < londist):
       ix = i
       londist = abs(lon_mod[i]-lon_draw)

print("draw along ", lon_mod[ix])

dic_tmp = np.array(np.empty((nz,ny,nx)),dtype=np.float64)
dic_prei_tmp = np.array(np.empty((nz,ny,nx)),dtype=np.float64)
cant_g100 = np.array(np.empty((nyr,nz,ny,nx)),dtype=np.float64)

for yr in range(styr,edyr+1):

    suffix = '.' + '{:04}'.format(yr)

    dic_file = path_g100 + '/' + base_dic_g100 + suffix
    ncg100 = netCDF4.Dataset(dic_file,'r')
    dic_tmp[:,:,:] = ncg100.variables['dissic'][0,:,:,:]
    ncg100.close()
    dic_tmp = np.where(dic_tmp < -9.0e33, np.NaN, dic_tmp)

    dic_prei_file = path_g100 + '/' + base_dic_prei_g100 + suffix
    ncg100 = netCDF4.Dataset(dic_prei_file,'r')
    dic_prei_tmp[:,:,:] = ncg100.variables['dissicnat'][0,:,:,:]
    ncg100.close()
    dic_prei_tmp = np.where(dic_prei_tmp < -9.0e33, np.NaN, dic_prei_tmp)

    cant_g100[yr-styr,:,:,:] = (dic_tmp[:,:,:] - dic_prei_tmp[:,:,:])*1e3

DS_g100_ann = xr.Dataset( {'cant': (['time','depth','lat','lon'], cant_g100),},
                          coords = { 'time': time, 'depth': dep_mod, 'lat': lat_mod,'lon': lon_mod, } )

######

base_sig0_g100 = 'hs_sigma0_woa1x1'

sig0_tmp = np.array(np.empty((nz,ny,nx)),dtype=np.float64)
sig0_g100 = np.array(np.empty((nyr,nz,ny,nx)),dtype=np.float64)

for yr in range(styr,edyr+1):

    suffix = '.' + '{:04}'.format(yr)

    sig0_file = path_g100 + '/' + base_sig0_g100 + suffix
    ncg100 = netCDF4.Dataset(sig0_file,'r')
    sig0_tmp[:,:,:] = ncg100.variables['sigma0'][0,:,:,:]
    ncg100.close()
    sig0_tmp = np.where(sig0_tmp < -9.0e33, np.NaN, sig0_tmp)
    sig0_g100[yr-styr,:,:,:] = sig0_tmp[:,:,:].copy()

DS_g100_sig0 = xr.Dataset( {'sigma0': (['time','depth','lat','lon'], sig0_g100),},
                          coords = { 'time': time, 'depth': dep_mod, 'lat': lat_mod,'lon': lon_mod, } )

####################################################################
# NP10km

print( "Loading NP10km data" )

path_fill=path_np10km + 'hst_ann-glb_fill'

cant_fill = np.array(np.empty((nyr,nz,ny,nx)),dtype=np.float64)

base_dic_fill = 'hs_DIC_woa1x1'
base_dic_prei_fill = 'hs_DIC_prei_woa1x1'

dic_tmp[:,:,:] = np.NaN
dic_prei_tmp[:,:,:] = np.NaN

for yr in range(styr,edyr+1):

    suffix = '.' + '{:04}'.format(yr)

    dic_file = path_fill + '/y' + '{:04}'.format(yr) + '/' + base_dic_fill + suffix
    ncfill = netCDF4.Dataset(dic_file,'r')
    dic_tmp[0:nz-1,:,:] = ncfill.variables['dissic'][0,0:nz-1,:,:]
    ncfill.close()
    dic_tmp = np.where(dic_tmp < -9.0e33, np.NaN, dic_tmp)

    dic_prei_file = path_fill + '/y' + '{:04}'.format(yr) + '/' + base_dic_prei_fill + suffix
    ncfill = netCDF4.Dataset(dic_prei_file,'r')
    dic_prei_tmp[0:nz-1,:,:] = ncfill.variables['dissicnat'][0,0:nz-1,:,:]
    ncfill.close()
    dic_prei_tmp = np.where(dic_prei_tmp < -9.0e33, np.NaN, dic_prei_tmp)

    cant_fill[yr-styr,:,:,:] = (dic_tmp[:,:,:] - dic_prei_tmp[:,:,:])*1e3

DS_fill_ann = xr.Dataset( {'cant': (['time','depth','lat','lon'], cant_fill),},
                          coords = { 'time': time, 'depth': dep_mod, 'lat': lat_mod,'lon': lon_mod, } )

#print(DS_fill_ann)

######

base_sig0_fill = 'hs_sigma0_woa1x1'

sig0_tmp = np.array(np.empty((nz,ny,nx)),dtype=np.float64)
sig0_fill = np.array(np.empty((nyr,nz,ny,nx)),dtype=np.float64)

for yr in range(styr,edyr+1):

    suffix = '.' + '{:04}'.format(yr)

    sig0_file = path_fill + '/y' + '{:04}'.format(yr) + '/' + base_sig0_fill + suffix
    ncfill = netCDF4.Dataset(sig0_file,'r')
    sig0_tmp[0:nz-1,:,:] = ncfill.variables['sigma0'][0,0:nz-1,:,:]
    ncfill.close()
    sig0_tmp = np.where(sig0_tmp < -9.0e33, np.NaN, sig0_tmp)
    sig0_fill[yr-styr,:,:,:] = sig0_tmp[:,:,:].copy()

DS_fill_sig0 = xr.Dataset( {'sigma0': (['time','depth','lat','lon'], sig0_fill),},
                          coords = { 'time': time, 'depth': dep_mod, 'lat': lat_mod,'lon': lon_mod, } )

####################################################################

bias = np.empty( (4,nz,ny,nx) )

da0 = DS_ref.cant.interp(depth=dep_mod)

da1 = DS_g100_ann.cant.sel(time=slice(str(styr),str(edyr))).mean(dim='time',skipna=False)
#bias[1] = da1.values - da0.values
bias[1] = da1.values

da2 = DS_fill_ann.cant.sel(time=slice(str(styr),str(edyr))).mean(dim='time',skipna=False)
#bias[2] = da2.values - da0.values
bias[2] = da2.values

bias[3] = da2.values - da1.values

DS = xr.Dataset( {'GLB100km': (['depth','lat','lon'], bias[1]),
                  'NP10km': (['depth','lat','lon'], bias[2]),
                  'NP10km-GLB100km': (['depth','lat','lon'], bias[3]),
                  'GLODAPv2':   (['depth','lat','lon'], da0.values), },
                 coords = { 'depth': dep_mod, 'lat': lat_mod, 'lon': lon_mod, } )

#                  'GLODAPv2':   (['lat','lon'], da0.values), },

######
suptitle = r'C$_{\mathrm{anth}}$ concentration and density ($\sigma_0$),  longitude = ' + str(lon_mod[ix]) + ' (ave. from '+str(styr)+' to '+str(edyr)+')'
outfile = './fig/Cant_comp-'+str(int(lon_draw))

print( 'Start drawing' )
fig = plt.figure(figsize=(11,8))
fig.suptitle( suptitle, fontsize=18 )

ax = [
    plt.subplot(2,2,1),
    plt.subplot(2,2,2),
    plt.subplot(2,2,3),
    plt.subplot(2,2,4),
]

# [left, bottom, width, height]
#ax_cbar = [
#    plt.axes([0.93,0.64,0.012,0.23]),
#    plt.axes([0.93,0.37,0.012,0.23]),
#    plt.axes([0.93,0.10,0.012,0.23]),
#]

#bounds1 = [0.0, 1.0, 2.0, 5.0, 10.0, 15.0, 20.0, 30.0, 40.0, 50.0, 70.0, 100.0]
#bounds2 = [-20.0, -10.0, -7.0, -4.0, -2.0, -1.0, 1.0, 2.0, 4.0, 7.0, 10.0, 20.0]
#bounds3 = [0.0, 1.0, 2.0, 5.0, 10.0, 15.0, 20.0, 30.0, 40.0, 50.0, 70.0, 100.0]

bounds1 = np.arange(0,70.1,5)
tbounds1 = np.arange(0,70.1,10)
bounds2 = [-20.0, -10.0, -7.0, -4.0, -2.0, -1.0, 1.0, 2.0, 4.0, 7.0, 10.0, 20.0]
bounds3 = np.arange(0,70.1,5)
tbounds3 = np.arange(0,70.1,10)


#cmap = [ 'RdBu_r', 'RdBu_r', 'bwr', 'YlOrRd' ]
cmap = [ 'YlOrRd', 'YlOrRd', 'bwr', 'YlOrRd' ]
item = [ 'GLB100km', 'NP10km', 'NP10km-GLB100km', 'GLODAPv2' ]

for panel in range(4):
    print("Drawing panel "+ str(panel))
    if (item[panel] == 'GLB100km'):
#        bounds = bounds2
#        ticks_bounds = bounds2
        bounds = bounds1
        ticks_bounds = tbounds1
        da = DS[item[panel]].isel(lon=ix).sel(depth=slice(0,1000))
    elif (item[panel] == 'NP10km'):
#        bounds = bounds2
#        ticks_bounds = bounds2
        bounds = bounds1
        ticks_bounds = tbounds1
        da = DS[item[panel]].isel(lon=ix).sel(depth=slice(0,1000))
    elif item[panel] == 'NP10km-GLB100km':
        bounds = bounds2
        ticks_bounds = bounds2
        da = DS[item[panel]].isel(lon=ix).sel(depth=slice(0,1000))
    else:
        #da = DS_ref['cant'].isel(lon=ix).sel(depth=slice(0,1000))
        bounds = bounds3
        ticks_bounds = tbounds3
        da = DS[item[panel]].isel(lon=ix).sel(depth=slice(0,1000))

    da.plot(ax=ax[panel],cmap=cmap[panel],
            levels=bounds,
            extend='both',
#            cbar_ax = ax_cbar[ii],
            cbar_kwargs={'orientation': 'horizontal',
#                         'spacing':'proportional',
                         'spacing':'uniform',
                         'shrink':0.95,
                         'label': '[$\mathrm{mmol}\, \mathrm{m}^{-3}$]',
                         'ticks': ticks_bounds, },)

    if (panel == 0 or panel == 2):
        da3 = DS_g100_sig0.sigma0.isel(lon=ix).sel(depth=slice(0,1000)).sel(time=slice(str(styr),str(edyr))).mean(dim='time',skipna=False)
        cont = da3.plot.contour(ax=ax[panel],
                                colors=['slateblue','blue','mediumblue','darkblue','navy','midnightblue'],
                                levels=[24.0,25.0,25.5,26.0,26.5,27.0],
                                add_colorbar=False)

        if (panel == 0):
            clevels=cont.levels
            cont.clabel(fmt='%.1f',fontsize=8,colors='black',inline_spacing=1)

    if (panel == 1 or panel == 2):
        da4 = DS_fill_sig0.sigma0.isel(lon=ix).sel(depth=slice(0,1000)).sel(time=slice(str(styr),str(edyr))).mean(dim='time',skipna=False)
        cont = da4.plot.contour(ax=ax[panel],
                                colors=['lightcoral','indianred','brown','firebrick','maroon','darkred'],
                                levels=[24.0,25.0,25.5,26.0,26.5,27.0],
                                add_colorbar=False)
        if (panel == 1):
            clevels=cont.levels
            cont.clabel(fmt='%.1f',fontsize=8,colors='black',inline_spacing=1)

    if (panel == 3):
        da4 = DS_sig0_ref.sigma0.isel(lon=ix).sel(depth=slice(0,1000))
        cont = da4.plot.contour(ax=ax[panel],
                                colors=['black'],
                                levels=[24.0,25.0,25.5,26.0,26.5,27.0],
                                add_colorbar=False)
        clevels=cont.levels
        cont.clabel(fmt='%.1f',fontsize=8,colors='black',inline_spacing=1)

#    else:
#        da.plot(ax=ax[panel],cmap=cmap[panel],
#            levels=bounds,
#            extend='both',
#            add_colorbar=False,
#            transform=ccrs.PlateCarree())

        
    ax[panel].grid(lw='0.5',ls='--')
    ax[panel].set_xlabel('latitude')
    ax[panel].set_ylabel('depth')
    ax[panel].set_title(title[panel],{'fontsize':12,'verticalalignment':'top'})
    ax[panel].tick_params(labelsize=10)
    ax[panel].invert_yaxis()
    ax[panel].set_xlim(xlim[panel][0],xlim[panel][1])
    ax[panel].set_xticks(np.arange(xlim[panel][0],xlim[panel][1]+0.1,10))
    ax[panel].set_facecolor('lightgray')

plt.subplots_adjust(left=0.07,right=0.98,bottom=0.05,top=0.92,wspace=0.16,hspace=0.15)

outpdf = outfile+'.pdf'
outpng = outfile+'.png'

plt.savefig(outpng, bbox_inches='tight', pad_inches=0.0)
plt.savefig(outpdf, bbox_inches='tight', pad_inches=0.0)

print("figure is saved to " + outpng + " and " + outpdf)

plt.show()
