# -*- coding: utf-8 -*-

import sys
import json
import numpy as np
import pandas as pd
import xarray as xr
import matplotlib.pyplot as plt
from sklearn.linear_model import LinearRegression

#-----------------------------------------------------------------------------
#if (len(sys.argv) < 2):
#    print ('Usage: ' + sys.argv[0] + ' region')
#    print ([ "(1) Tropics", "(2) SW-Subtropical", "(3) NW-Subtropical", "(4) E-Subtropical", "(5) Subarctic", "(6) JapanSea", "(7) OkhotskSea" ])

#    sys.exit()

#region = int(sys.argv[1]) - 1

#-----------------------------------------------------------------------------

mipcols = ["darkred", "darkblue"]
miparcs = ["orange", "lightblue"]

title_list = [ "(a) Tropics",
               "(b) SW-Subtropical", 
               "(c) NW-Subtropical", 
               "(d) E-Subtropical", 
               "(e) Subarctic",
               "(f) Japan Sea", 
               "(g) Okhotsk sea",
               ]

#-----------------------------------------------------------------------------

fig = plt.figure(figsize=(11,8.5))

# [left, bottom, width, height]

ax=[ plt.axes([0.07,0.74,0.18,0.16]),
     plt.axes([0.31,0.74,0.18,0.16]),
     plt.axes([0.55,0.74,0.18,0.16]),
     plt.axes([0.79,0.74,0.18,0.16]),
     plt.axes([0.07,0.51,0.18,0.16]),
     plt.axes([0.31,0.51,0.18,0.16]),
     plt.axes([0.55,0.51,0.18,0.16]),
     plt.axes([0.79,0.51,0.18,0.16]),
     plt.axes([0.07,0.28,0.18,0.16]),
     plt.axes([0.31,0.28,0.18,0.16]),
     plt.axes([0.55,0.28,0.18,0.16]),
     plt.axes([0.79,0.28,0.18,0.16]),
     plt.axes([0.07,0.05,0.18,0.16]),
     plt.axes([0.31,0.05,0.18,0.16]),
     plt.axes([0.55,0.05,0.18,0.16]),
     plt.axes([0.79,0.05,0.18,0.16]), ]

######
path_glb100km='../data/glb100km/run-20211104o2/'
path_np10km='../data/np10km/'
######
# storage rate

nax=0
region=3
item_list = [ "Tropics", "SW-Subtropical", "NW-Subtropical", "E-Subtropical", "Subarctic", 
              "JapanSea", "OkhotskSea" ]

glb_list = [ "GLB-TRP", "GLB-SWST", "GLB-NWST", "GLB-EST", "GLB-SAR", "GLB-JPS", "GLB-OKH" ]
np_list = [ "NP-TRP", "NP-SWST", "NP-NWST", "NP-EST", "NP-SAR", "NP-JPS", "NP-OKH" ]

fig.suptitle(r'C$_{\mathrm{anth}}$ storage rate (1987-2017) summary for '+item_list[region])

infl1 = path_glb100km + "csv/Pac3_Cant_storage_rate_lyr_GLB100km_1987-2017.csv"
df1=pd.read_csv(infl1,index_col=0)
df1.set_axis(pd.Index(glb_list),axis=1,inplace=True)

infl4 = path_np10km + "csv/Pac3_Cant_storage_rate_lyr_NP10km_1987-2017.csv"
df4=pd.read_csv(infl4,index_col=0)
df4.set_axis(pd.Index(np_list),axis=1,inplace=True)

dfa=pd.concat([df1,df4],axis=1)

print(dfa)

dfa[[glb_list[region],np_list[region]]].iloc[0:10].plot.bar(ax=ax[nax],subplots=False)
total_glb = dfa[glb_list[region]].sum(axis=0)
total_np = dfa[np_list[region]].sum(axis=0)
print(total_glb,total_np)
ax[nax].set_title('(a) Storage rate'+'\n GLB100km ='+'{:.1f}'.format(total_glb) + ',  NP10km = '+'{:.1f}'.format(total_np),{'fontsize':8},pad=2)
#ax[nax].set_title('(a) Storage rate',{'fontsize':8})
ax[nax].set_xlabel('')
ax[nax].set_ylabel('[$\mathrm{TgC}\,\mathrm{yr}^{-1}$]',fontsize=8)
ax[nax].set_xticklabels(dfa.index.values[0:10],rotation=35)
ax[nax].set_ylim([-2,22])
ax[nax].tick_params(labelsize=7)
ax[nax].legend(fontsize=7)
#ax[nax].set_ylim(cnvrange_list[item][0],cnvrange_list[item][1])

##############
# Surface flux

nax=1
region=6

item_list = [ "Tropics", "SW-Subtropical", "NW-Subtropical",
              "Subarctic", "JapanSea", "OkhotskSea", "E-Subtropical" ]

glb_list = [ "GLB-TRP", "GLB-SWST", "GLB-NWST", "GLB-SAR", "GLB-JPS", "GLB-OKH", "GLB-EST" ]
np_list = [ "NP-TRP", "NP-SWST", "NP-NWST", "NP-SAR", "NP-JPS", "NP-OKH", "NP-EST" ]

#-----------------------------------------------------------------------------

infl1 = path_glb100km + "csv/Pac3_Cant_flux_lyr_GLB100km_1987-2017.csv"
df1=pd.read_csv(infl1,index_col=0)
infl2 = path_glb100km + "csv/Pac3_Cant_flux_v_lyr_GLB100km_1987-2017.csv"
df2=pd.read_csv(infl2,index_col=0)

df1.set_axis(pd.Index(glb_list),axis=1,inplace=True)
df2.set_axis(pd.Index(glb_list),axis=1,inplace=True)

infl4 = path_np10km + "csv/Pac3_Cant_flux_lyr_NP10km_1987-2017.csv"
df4=pd.read_csv(infl4,index_col=0)
infl5 = path_np10km + "csv/Pac3_Cant_flux_v_lyr_NP10km_1987-2017.csv"
df5=pd.read_csv(infl5,index_col=0)

df4.set_axis(pd.Index(np_list),axis=1,inplace=True)
df5.set_axis(pd.Index(np_list),axis=1,inplace=True)

dfa=df1+df2
dfb=df4+df5

dfc=pd.concat([dfa,dfb],axis=1)
print(dfc)
#----------------------

dfc[[glb_list[region],np_list[region]]].iloc[0:10].plot.bar(ax=ax[nax],subplots=False)
total_glb = dfc[glb_list[region]].sum(axis=0)
total_np = dfc[np_list[region]].sum(axis=0)
print(total_glb,total_np)
ax[nax].set_title('(b) Surface uptake'+'\n GLB100km ='+'{:.1f}'.format(total_glb) + ',   NP10km = '+'{:.1f}'.format(total_np),fontsize=8,pad=2)
ax[nax].set_xlabel('')
ax[nax].set_ylabel('[$\mathrm{TgC}\,\mathrm{yr}^{-1}$]',fontsize=8)
#ax[nax].set_title('(b) Surface uptake',fontsize=8)
ax[nax].set_xticklabels(dfa.index.values[0:10],rotation=35)
ax[nax].set_ylim([-2,22])
ax[nax].tick_params(labelsize=7)
ax[nax].legend(fontsize=7)
#axes.set_ylim(cnvrange_list[region][0],cnvrange_list[region][1])

##############
# Convergence

nax=2
region=3

item_list = [ "Tropics", "SW-Subtropical", "NW-Subtropical", "E-Subtropical", "Subarctic", 
              "JapanSea", "OkhotskSea" ]

glb_list = [ "GLB-TRP", "GLB-SWST", "GLB-NWST", "GLB-EST", "GLB-SAR", "GLB-JPS", "GLB-OKH" ]
np_list = [ "NP-TRP", "NP-SWST", "NP-NWST", "NP-EST", "NP-SAR", "NP-JPS", "NP-OKH" ]


infl1 = path_glb100km + "csv/Pac3_Cant_convergence_lyr_GLB100km_1987-2017.csv"
df1=pd.read_csv(infl1,index_col=0)
df1.set_axis(pd.Index(glb_list),axis=1,inplace=True)

infl4 = path_np10km + "csv/Pac3_Cant_convergence_lyr_NP10km_1987-2017.csv"
df4=pd.read_csv(infl4,index_col=0)
df4.set_axis(pd.Index(np_list),axis=1,inplace=True)

dfa=pd.concat([df1,df4],axis=1)

print(dfa)
#----------------------

dfa[[glb_list[region],np_list[region]]].iloc[0:10].plot.bar(ax=ax[nax],subplots=False)
total_glb = dfa[glb_list[region]].sum(axis=0)
total_np = dfa[np_list[region]].sum(axis=0)
print(total_glb,total_np)
ax[nax].set_title('(c) Transport Convergence'+'\n GLB100km ='+'{:.1f}'.format(total_glb) + ',   NP10km = '+'{:.1f}'.format(total_np),fontsize=8,pad=2)
#ax[nax].set_title('(c) Transport Convergence',fontsize=8)
ax[nax].set_xticklabels(dfa.index.values[0:10],rotation=35)
ax[nax].tick_params(labelsize=7)
ax[nax].set_xlabel('')
ax[nax].set_ylabel(r'[$\mathrm{TgC}\,\mathrm{yr}^{-1}$]',fontsize=8)
ax[nax].set_ylim([-110,110])
ax[nax].legend(fontsize=7)
#axes.set_ylim(cnvrange_list[item][0],cnvrange_list[item][1])
################
# conversion

nax=3
region=3

item_list = [ "Tropics", "SW-Subtropical", "NW-Subtropical", "E-Subtropical", "Subarctic", 
              "JapanSea", "OkhotskSea" ]

glb_list = [ "GLB-TRP", "GLB-SWST", "GLB-NWST", "GLB-EST", "GLB-SAR", "GLB-JPS", "GLB-OKH" ]
np_list = [ "NP-TRP", "NP-SWST", "NP-NWST", "NP-EST", "NP-SAR", "NP-JPS", "NP-OKH" ]

#-----------------------------------------------------------------------------

infl1 = path_glb100km + "csv/Pac3_Cant_conversion_lyr_GLB100km_1987-2017.csv"
df1=pd.read_csv(infl1,index_col=0)
df1.set_axis(pd.Index(glb_list),axis=1,inplace=True)

infl4 = path_np10km + "csv/Pac3_Cant_conversion_lyr_NP10km_1987-2017.csv"
df4=pd.read_csv(infl4,index_col=0)
df4.set_axis(pd.Index(np_list),axis=1,inplace=True)

df_conv_all=pd.concat([df1,df4],axis=1)

print(df_conv_all)

df_conv_all[[glb_list[region],np_list[region]]].iloc[0:10].plot.bar(ax=ax[nax],subplots=False)
ax[nax].set_title('(d) Conversion rate',fontsize=8)
ax[nax].set_xticklabels(df_conv_all.index.values[0:10],rotation=35)
ax[nax].tick_params(labelsize=7)
ax[nax].set_xlabel('')
ax[nax].set_ylabel(r'[$\mathrm{TgC}\,\mathrm{yr}^{-1}$]',fontsize=8)
ax[nax].set_ylim([-110,110])
ax[nax].legend(fontsize=7)
#ax[nax].set_ylim(cnvrange_list[item][0],cnvrange_list[item][1])

######################
# transports

item_list = [ "Tsushima", "Tsugaru", "Soya", "Tatar", "ITF",
              "Bering", "Lat12S", "Lat12Nw", "Lat28Nw","Lat42Nw", 
              "Kuril", "Lon170Wn", "Lon170Ws", "Lat42Ne", "Lat12Ne" ]

glb_list = [ "GLB-TSM", "GLB-TGR", "GLB-SOY", "GLB-TTR", "GLB-ITF",
             "GLB-BER", "GLB-12S", "GLB-12Nw", "GLB-28Nw", "GLB-42Nw",
             "GLB-KRL", "GLB-170Wn","GLB-170Ws", "GLB-42Ne","GLB-12Ne" ]

np_list = [ "NP-TSM", "NP-TGR", "NP-SOY", "NP-TTR", "NP-ITF",
            "NP-BER", "NP-12S", "NP-12Nw", "NP-28Nw", "NP-42Nw",
            "NP-KRL", "NP-170Wn", "NP-170Ws", "NP-42Ne", "NP-12Ne" ]

sign_list = [ -1.0,  1.0, -1.0, -1.0,  1.0,
              -1.0,  1.0, -1.0, -1.0, -1.0,
               1.0, -1.0, -1.0,  1.0, -1.0 ]

range_list = [ [-5.0, 20.0], [-5.0, 20.0], [-5.0, 20.0], [-5.0, 20.0], [-5.0, 20.0],
               [-5.0, 20.0], [-250.0, 120.0], [-120.0, 250.0], [-120.0, 250.0], [-50.0, 50.0],
               [-50.0, 50.0], [-200.0, 200.0], [-200.0, 200.0], [-50.0, 100.0], [-120.0, 250.0] ]


#############################################
# output mean to CSV file 

infl1=path_glb100km+'csv/Pac3_Cant_trans_lyr_resolved_GLB100km_1987-2017.csv'
df_vlg=pd.read_csv(infl1,index_col=0)
df_vlg.set_axis(pd.Index(glb_list),axis=1,inplace=True)

infl2=path_glb100km+'csv/Pac3_Cant_trans_lyr_gm_GLB100km_1987-2017.csv'
df_gmg=pd.read_csv(infl2,index_col=0)
df_gmg.set_axis(pd.Index(glb_list),axis=1,inplace=True)

infl3=path_glb100km+'csv/Pac3_Cant_trans_lyr_iso_GLB100km_1987-2017.csv'
df_isg=pd.read_csv(infl3,index_col=0)
df_isg.set_axis(pd.Index(glb_list),axis=1,inplace=True)

#####

infl5=path_np10km+'csv/Pac3_Cant_trans_lyr_resolved_NP10km_1987-2017.csv'
df_vln=pd.read_csv(infl5,index_col=0)
df_vln.set_axis(pd.Index(np_list),axis=1,inplace=True)

infl6=path_np10km+'csv/Pac3_Cant_trans_lyr_gm_NP10km_1987-2017.csv'
df_gmn=pd.read_csv(infl6,index_col=0)
df_gmn.set_axis(pd.Index(np_list),axis=1,inplace=True)

infl7=path_np10km+'csv/Pac3_Cant_trans_lyr_iso_NP10km_1987-2017.csv'
df_isn=pd.read_csv(infl7,index_col=0)
df_isn.set_axis(pd.Index(np_list),axis=1,inplace=True)

infl8=path_np10km+'csv/Pac3_Cant_trans_lyr_hdif_NP10km_1987-2017.csv'
df_hdn=pd.read_csv(infl8,index_col=0)
df_hdn.set_axis(pd.Index(np_list),axis=1,inplace=True)

dfa=pd.concat([df_vlg+df_gmg+df_isg,df_vln+df_gmn+df_isn+df_hdn],axis=1)

print(dfa)

###################################################

#total_glb = dfb[glb_list[region]].sum(axis=0)
#total_np = dfb[np_list[region]].sum(axis=0)
#print(total_glb,total_np)

nax=4
region=11
dfb = dfa[[glb_list[region],np_list[region]]] * sign_list[region]

dfb[[glb_list[region],np_list[region]]].iloc[0:10].plot.bar(ax=ax[nax],subplots=False)

total_glb = dfb[glb_list[region]].sum(axis=0)
total_np = dfb[np_list[region]].sum(axis=0)
print(total_glb,total_np)
ax[nax].set_title('(e) Transport '+item_list[region]+'\n GLB100km ='+'{:.1f}'.format(total_glb) + ',  NP10km = '+'{:.1f}'.format(total_np),fontsize=8,pad=2)
#ax[nax].set_title('(e) Transport '+item_list[region],fontsize=8)
ax[nax].set_xticklabels(dfa.index.values[0:10],rotation=35)
ax[nax].tick_params(labelsize=7)
ax[nax].set_xlabel('')
ax[nax].set_ylabel(r'[$\mathrm{TgC}\,\mathrm{yr}^{-1}$]',fontsize=8)
ax[nax].set_ylim([-105,150])
ax[nax].legend(fontsize=7)
#ax[nax].set_ylim(cnvrange_list[item][0],cnvrange_list[item][1])

nax=5
region=12
dfb = dfa[[glb_list[region],np_list[region]]] * sign_list[region]

dfb[[glb_list[region],np_list[region]]].iloc[0:10].plot.bar(ax=ax[nax],subplots=False)
total_glb = dfb[glb_list[region]].sum(axis=0)
total_np = dfb[np_list[region]].sum(axis=0)
print(total_glb,total_np)
ax[nax].set_title('(f) Transport '+item_list[region]+'\n GLB100km ='+'{:.1f}'.format(total_glb) + ',   NP10km = '+'{:.1f}'.format(total_np),fontsize=8,pad=2)
#ax[nax].set_title('(f) Transport '+item_list[region],fontsize=8)
ax[nax].set_xticklabels(dfa.index.values[0:10],rotation=35)
ax[nax].tick_params(labelsize=7)
ax[nax].set_xlabel('')
ax[nax].set_ylabel(r'[$\mathrm{TgC}\,\mathrm{yr}^{-1}$]',fontsize=8)
ax[nax].set_ylim([-105,150])
ax[nax].legend(fontsize=7)
#ax[nax].set_ylim(cnvrange_list[item][0],cnvrange_list[item][1])

nax=6
region=13
dfb = -dfa[[glb_list[region],np_list[region]]] * sign_list[region]

dfb[[glb_list[region],np_list[region]]].iloc[0:10].plot.bar(ax=ax[nax],subplots=False)
total_glb = dfb[glb_list[region]].sum(axis=0)
total_np = dfb[np_list[region]].sum(axis=0)
print(total_glb,total_np)
ax[nax].set_title('(g) Transport '+item_list[region]+'\n GLB100km ='+'{:.1f}'.format(total_glb) + ',   NP10km = '+'{:.1f}'.format(total_np),fontsize=8,pad=2)
#ax[nax].set_title('(g) Transport '+item_list[region],fontsize=8)
ax[nax].set_xticklabels(dfa.index.values[0:10],rotation=35)
ax[nax].tick_params(labelsize=7)
ax[nax].set_xlabel('')
ax[nax].set_ylabel(r'[$\mathrm{TgC}\,\mathrm{yr}^{-1}$]',fontsize=8)
ax[nax].set_ylim([-105,150])
ax[nax].legend(fontsize=7)
#ax[nax].set_ylim(cnvrange_list[item][0],cnvrange_list[item][1])

nax=7
region=14
dfb = dfa[[glb_list[region],np_list[region]]] * sign_list[region]

dfb[[glb_list[region],np_list[region]]].iloc[0:10].plot.bar(ax=ax[nax],subplots=False)
total_glb = dfb[glb_list[region]].sum(axis=0)
total_np = dfb[np_list[region]].sum(axis=0)
print(total_glb,total_np)
ax[nax].set_title('(h) Transport '+item_list[region]+'\n GLB100km ='+'{:.1f}'.format(total_glb) + ',   NP10km = '+'{:.1f}'.format(total_np),fontsize=8,pad=2)
#ax[nax].set_title('(g) Transport '+item_list[region],fontsize=8)
ax[nax].set_xticklabels(dfa.index.values[0:10],rotation=35)
ax[nax].tick_params(labelsize=7)
ax[nax].set_xlabel('')
ax[nax].set_ylabel(r'[$\mathrm{TgC}\,\mathrm{yr}^{-1}$]',fontsize=8)
ax[nax].set_ylim([-105,150])
ax[nax].legend(fontsize=7)
#ax[nax].set_ylim(cnvrange_list[item][0],cnvrange_list[item][1])



##########
# explicit density conversion

nax=8
region=6

item_list = [ "Tropics", "SW-Subtropical", "NW-Subtropical", "Subarctic", 
              "JapanSea", "OkhotskSea", "E-Subtropical" ]

glb_list = [ "GLB-TRP", "GLB-SWST", "GLB-NWST", "GLB-SAR", "GLB-JPS", "GLB-OKH", "GLB-EST" ]
np_list = [ "NP-TRP", "NP-SWST", "NP-NWST", "NP-SAR", "NP-JPS", "NP-OKH", "NP-EST" ]

# explicit mixing

infl1 = path_glb100km+"ANALY/annclim/hs_pac3_e_sig_exp_Cant_lyr_anncl.1987-2017.nc"
infl2 = path_np10km+"hst_annclim-np/hs_pac3_e_sig_exp_Cant_lyr_anncl.1987-2017.nc"

figa_title='diapyc transp. (explicit all) '
figb_title='diapyc transp. conv. (explicit all) '
sign=1.0
ds_dpy1 = xr.open_dataset(infl1)
ds_dpy1_slim=ds_dpy1.drop_dims(['lat','lon'])
df_dpy1=sign*ds_dpy1_slim.mean(dim='time').to_dataframe()*86400*365.25*1e-12
print(df_dpy1)

ds_dpy2 = xr.open_dataset(infl2)
ds_dpy2_slim=ds_dpy2.drop_dims(['lat','lon'])
df_dpy2=sign*ds_dpy2_slim.mean(dim='time').to_dataframe()*86400*365.25*1e-12
print(df_dpy2)


df_dpy1.set_axis(pd.Index(glb_list),axis=1,inplace=True)
df_dpy2.set_axis(pd.Index(np_list),axis=1,inplace=True)

df_dpy=pd.concat([df_dpy1,df_dpy2],axis=1)
print(df_dpy)

#######

dpy1_flux=df_dpy1.to_numpy()
nz,nx=dpy1_flux.shape
#print(nz,nx)
dpy1_conv=np.array(np.zeros((nz,nx)),dtype=np.float64)

for i in range(nx):
    dpy1_conv[0,i]=dpy1_flux[0,i]

for k in range(1,nz):
    for i in range(nx):
        dpy1_conv[k,i]=dpy1_flux[k,i]-dpy1_flux[k-1,i]

dpy2_flux=df_dpy2.to_numpy()
nz,nx=dpy2_flux.shape
#print(nz,nx)
dpy2_conv=np.array(np.zeros((nz,nx)),dtype=np.float64)

for i in range(nx):
    dpy2_conv[0,i]=dpy2_flux[0,i]

for k in range(1,nz):
    for i in range(nx):
        dpy2_conv[k,i]=dpy2_flux[k,i]-dpy2_flux[k-1,i]

den_half = [21.5,22.5,23.5,24.25,24.75,25.25,25.75,26.25,26.75,27.25]

denidx=pd.Index(data=den_half,name="density")
basidx=pd.Index(data=item_list,name="region")

df1_conv=pd.DataFrame(data=dpy1_conv,index=denidx,columns=basidx)
df2_conv=pd.DataFrame(data=dpy2_conv,index=denidx,columns=basidx)

df1_conv.set_axis(pd.Index(glb_list),axis=1,inplace=True)
df2_conv.set_axis(pd.Index(np_list),axis=1,inplace=True)

df_conv=pd.concat([df1_conv,df2_conv],axis=1)
print(df_conv)

#----------------------

df_conv[[glb_list[region],np_list[region]]].iloc[0:10].plot.bar(ax=ax[nax],subplots=False)
ax[nax].set_title('(i) '+figb_title,fontsize=8)
ax[nax].set_xlabel('')
ax[nax].set_ylabel(r'[$\mathrm{TgC}\,\mathrm{yr}^{-1}$]',fontsize=8)
ax[nax].set_xticklabels(dfa.index.values[0:10],rotation=35)
ax[nax].tick_params(labelsize=7)
ax[nax].set_ylim([-110,110])
ax[nax].legend(fontsize=7)
#ax[nax].set_xlim(21.0,28.0)
#ax[nax].set_ylim(cnvrange_list[item][0],cnvrange_list[item][1])

##########
# vertical diffusion

nax=10
region=6

item_list = [ "Tropics", "SW-Subtropical", "NW-Subtropical", "Subarctic", 
              "JapanSea", "OkhotskSea", "E-Subtropical" ]

glb_list = [ "GLB-TRP", "GLB-SWST", "GLB-NWST", "GLB-SAR", "GLB-JPS", "GLB-OKH", "GLB-EST" ]
np_list = [ "NP-TRP", "NP-SWST", "NP-NWST", "NP-SAR", "NP-JPS", "NP-OKH", "NP-EST" ]

#-----------------------------------------------------------------------------

infl1 = path_glb100km+"csv/Pac3_Cant_storage_vdif_lyr_GLB100km_1987-2017.csv"
df1=pd.read_csv(infl1,index_col=0)
df1.set_axis(pd.Index(glb_list),axis=1,inplace=True)

infl4 = path_np10km+"csv/Pac3_Cant_storage_vdif_lyr_NP10km_1987-2017.csv"
df4=pd.read_csv(infl4,index_col=0)
df4.set_axis(pd.Index(np_list),axis=1,inplace=True)

df_vdif=pd.concat([df1,df4],axis=1)

print(df_vdif)
#----------------------

#total_glb = dfa[glb_list[region]].sum(axis=0)
#total_np = dfa[np_list[region]].sum(axis=0)
#print(total_glb,total_np)

df_vdif[[glb_list[region],np_list[region]]].iloc[0:10].plot.bar(ax=ax[nax],subplots=False)
#axes.set_title('GLB100km ='+'{:.1f}'.format(total_glb) + ',   NP10km = '+'{:.1f}'.format(total_np) + '  [$\,\mathrm{TgC}\,\mathrm{yr}^{-1}$]')
ax[nax].set_title("(k) vertical diff ",fontsize=8)
ax[nax].set_xlabel('')
ax[nax].set_ylabel(r'[$\mathrm{TgC}\,\mathrm{yr}^{-1}$]',fontsize=8)
ax[nax].set_xticklabels(df_vdif.index.values[0:10],rotation=35)
ax[nax].tick_params(labelsize=7)
ax[nax].set_ylim([-20,20])
ax[nax].legend(fontsize=7)
#axes.set_ylim(cnvrange_list[item][0],cnvrange_list[item][1])

########
# GM scheme

#nax = 10
#region = 2

item_list = [ "Tropics", "SW-Subtropical", "NW-Subtropical", "Subarctic", 
              "JapanSea", "OkhotskSea", "E-Subtropical" ]

glb_list = [ "GLB-TRP", "GLB-SWST", "GLB-NWST", "GLB-SAR", "GLB-JPS", "GLB-OKH", "GLB-EST" ]
np_list = [ "NP-TRP", "NP-SWST", "NP-NWST", "NP-SAR", "NP-JPS", "NP-OKH", "NP-EST" ]

csv_fname=path_glb100km+'csv/Pac3_Cant_storage_gm_minus_trans_lyr_GLB100km_1987-2017.csv'
dfg_conv_all=pd.read_csv(csv_fname,index_col=0)

csv_fname=path_np10km+'csv/Pac3_Cant_storage_gm_minus_trans_lyr_NP10km_1987-2017.csv'
dfn_conv_all=pd.read_csv(csv_fname,index_col=0)

df_conv_gm=pd.concat([dfg_conv_all,dfn_conv_all],axis=1)
print(df_conv_gm)

#df_conv_gm[[glb_list[region],np_list[region]]].iloc[0:10].plot.bar(ax=ax[nax],subplots=False)
#ax[nax].set_title('(k) GM skew flux '+item_list[region],fontsize=8)
#ax[nax].set_xlabel('')
#ax[nax].set_ylabel('[$\mathrm{TgC}\,\mathrm{yr}^{-1}$]',fontsize=8)
#ax[nax].set_xticklabels(dfa.index.values[0:10],rotation=35)
#ax[nax].tick_params(labelsize=7)
#ax[nax].set_ylim([-20,20])
#ax[nax].legend(fontsize=7)
#####ax[nax].set_ylim(cnvrange_list[item][0],cnvrange_list[item][1])

#######
# isopycnal scheme
nax=11
region=6

csv_fname=path_glb100km+'csv/Pac3_Cant_storage_iso_minus_trans_lyr_GLB100km_1987-2017.csv'
dfg_conv_all=pd.read_csv(csv_fname,index_col=0)

csv_fname=path_np10km+'csv/Pac3_Cant_storage_iso_minus_trans_lyr_NP10km_1987-2017.csv'
dfn_conv_all=pd.read_csv(csv_fname,index_col=0)

df_conv_iso=pd.concat([dfg_conv_all,dfn_conv_all],axis=1)
print(df_conv_iso)

df_conv_gm_iso = df_conv_gm + df_conv_iso

#df_conv_iso[[glb_list[region],np_list[region]]].iloc[0:10].plot.bar(ax=ax[nax],subplots=False)
df_conv_gm_iso[[glb_list[region],np_list[region]]].iloc[0:10].plot.bar(ax=ax[nax],subplots=False)
ax[nax].set_title('(l) GM skew flux + isopyc diff ' ,fontsize=8)
ax[nax].set_xlabel('')
ax[nax].set_ylabel('[$\mathrm{TgC}\,\mathrm{yr}^{-1}$]',fontsize=8)
ax[nax].set_xticklabels(dfa.index.values[0:10],rotation=35)
ax[nax].tick_params(labelsize=7)
ax[nax].set_ylim([-20,20])
ax[nax].legend(fontsize=7)


#axes.set_ylim(cnvrange_list[item][0],cnvrange_list[item][1])

################################
# gm diapycnal transport
nax = 9
region = 6

item_list = [ "Tropics", "SW-Subtropical", "NW-Subtropical", "Subarctic", 
              "JapanSea", "OkhotskSea", "E-Subtropical" ]

glb_list = [ "GLB-TRP", "GLB-SWST", "GLB-NWST", "GLB-SAR", "GLB-JPS", "GLB-OKH", "GLB-EST" ]
np_list = [ "NP-TRP", "NP-SWST", "NP-NWST", "NP-SAR", "NP-JPS", "NP-OKH", "NP-EST" ]

infl1 = path_glb100km+"ANALY/annclim/hs_pac3_e_sig_gm_Cant_lyr_anncl.1987-2017.nc"
infl2 = path_np10km+"hst_annclim-np/hs_pac3_e_sig_gm_Cant_lyr_anncl.1987-2017.nc"
fig_title='Cant diapycnal flux (gm) summary (1987-2017) for '+item_list[region]
figa_title='diapyc transp. (gm) '
figb_title='diapyc transp. conv. (gm) '
sign=1.0

ds_dpy1 = xr.open_dataset(infl1)
ds_dpy1_slim=ds_dpy1.drop_dims(['lat','lon'])
df_dpy1=sign*ds_dpy1_slim.mean(dim='time').to_dataframe()*86400*365.25*1e-12
print(df_dpy1)

ds_dpy2 = xr.open_dataset(infl2)
ds_dpy2_slim=ds_dpy2.drop_dims(['lat','lon'])
df_dpy2=sign*ds_dpy2_slim.mean(dim='time').to_dataframe()*86400*365.25*1e-12
print(df_dpy2)


df_dpy1.set_axis(pd.Index(glb_list),axis=1,inplace=True)
df_dpy2.set_axis(pd.Index(np_list),axis=1,inplace=True)

df_dpy=pd.concat([df_dpy1,df_dpy2],axis=1)
print(df_dpy)

#######

dpy1_flux=df_dpy1.to_numpy()
nz,nx=dpy1_flux.shape
#print(nz,nx)
dpy1_conv=np.array(np.zeros((nz,nx)),dtype=np.float64)

for i in range(nx):
    dpy1_conv[0,i]=dpy1_flux[0,i]

for k in range(1,nz):
    for i in range(nx):
        dpy1_conv[k,i]=dpy1_flux[k,i]-dpy1_flux[k-1,i]

dpy2_flux=df_dpy2.to_numpy()
nz,nx=dpy2_flux.shape
#print(nz,nx)
dpy2_conv=np.array(np.zeros((nz,nx)),dtype=np.float64)

for i in range(nx):
    dpy2_conv[0,i]=dpy2_flux[0,i]

for k in range(1,nz):
    for i in range(nx):
        dpy2_conv[k,i]=dpy2_flux[k,i]-dpy2_flux[k-1,i]

den_half = [21.5,22.5,23.5,24.25,24.75,25.25,25.75,26.25,26.75,27.25]

denidx=pd.Index(data=den_half,name="density")
basidx=pd.Index(data=item_list,name="region")

df1_conv=pd.DataFrame(data=dpy1_conv,index=denidx,columns=basidx)
df2_conv=pd.DataFrame(data=dpy2_conv,index=denidx,columns=basidx)

df1_conv.set_axis(pd.Index(glb_list),axis=1,inplace=True)
df2_conv.set_axis(pd.Index(np_list),axis=1,inplace=True)

df_conv_dpgm=pd.concat([df1_conv,df2_conv],axis=1)
print(df_conv_dpgm)

#----------------------

df_conv_dpgm[[glb_list[region],np_list[region]]].iloc[0:10].plot.bar(ax=ax[nax],subplots=False)
ax[nax].set_title('(j) '+figb_title ,fontsize=8)
ax[nax].set_xlabel('')
ax[nax].set_ylabel(r'[$\mathrm{TgC}\,\mathrm{yr}^{-1}$]',fontsize=8)
ax[nax].set_xticklabels(df_conv_dpgm.index.values[0:10],rotation=35)
ax[nax].tick_params(labelsize=7)
ax[nax].set_ylim([-20,20])
ax[nax].legend(fontsize=7)

##########
# residual
nax=12
region=6

df_conv_res=df_conv_all-df_conv-df_vdif-df_conv_gm_iso-df_conv_dpgm
print(df_conv_res)

df_conv_res[[glb_list[region],np_list[region]]].iloc[0:10].plot.bar(ax=ax[nax],subplots=False)
ax[nax].set_title('(m) residual of conversion rate',fontsize=8)
ax[nax].set_xlabel('')
ax[nax].set_ylabel(r'[$\mathrm{TgC}\,\mathrm{yr}^{-1}$]',fontsize=8)
ax[nax].set_xticklabels(dfa.index.values[0:10],rotation=35)
ax[nax].tick_params(labelsize=7)
ax[nax].set_ylim([-110,110])
ax[nax].legend(fontsize=7)

################################
#
nax=13
region=6

infl1 = path_glb100km+"ANALY/annclim/hs_pac3_e_sig_vdif_Cant_lyr_anncl.1987-2017.nc"
infl2 = path_np10km+"hst_annclim-np/hs_pac3_e_sig_vdif_Cant_lyr_anncl.1987-2017.nc"
fig_title='Cant diapycnal flux (vdif) summary (1987-2017) for '+item_list[region]
figa_title='diapyc transp. (explicit vdif) '
figb_title='diapyc transp. conv. (explicit vdif) '
sign=1.0

ds_dpy1 = xr.open_dataset(infl1)
ds_dpy1_slim=ds_dpy1.drop_dims(['lat','lon'])
df_dpy1=sign*ds_dpy1_slim.mean(dim='time').to_dataframe()*86400*365.25*1e-12
print(df_dpy1)

ds_dpy2 = xr.open_dataset(infl2)
ds_dpy2_slim=ds_dpy2.drop_dims(['lat','lon'])
df_dpy2=sign*ds_dpy2_slim.mean(dim='time').to_dataframe()*86400*365.25*1e-12
print(df_dpy2)


df_dpy1.set_axis(pd.Index(glb_list),axis=1,inplace=True)
df_dpy2.set_axis(pd.Index(np_list),axis=1,inplace=True)

df_dpy=pd.concat([df_dpy1,df_dpy2],axis=1)
print(df_dpy)

#######

dpy1_flux=df_dpy1.to_numpy()
nz,nx=dpy1_flux.shape
#print(nz,nx)
dpy1_conv=np.array(np.zeros((nz,nx)),dtype=np.float64)

for i in range(nx):
    dpy1_conv[0,i]=dpy1_flux[0,i]

for k in range(1,nz):
    for i in range(nx):
        dpy1_conv[k,i]=dpy1_flux[k,i]-dpy1_flux[k-1,i]

dpy2_flux=df_dpy2.to_numpy()
nz,nx=dpy2_flux.shape
#print(nz,nx)
dpy2_conv=np.array(np.zeros((nz,nx)),dtype=np.float64)

for i in range(nx):
    dpy2_conv[0,i]=dpy2_flux[0,i]

for k in range(1,nz):
    for i in range(nx):
        dpy2_conv[k,i]=dpy2_flux[k,i]-dpy2_flux[k-1,i]

den_half = [21.5,22.5,23.5,24.25,24.75,25.25,25.75,26.25,26.75,27.25]

denidx=pd.Index(data=den_half,name="density")
basidx=pd.Index(data=item_list,name="region")

df1_conv=pd.DataFrame(data=dpy1_conv,index=denidx,columns=basidx)
df2_conv=pd.DataFrame(data=dpy2_conv,index=denidx,columns=basidx)

df1_conv.set_axis(pd.Index(glb_list),axis=1,inplace=True)
df2_conv.set_axis(pd.Index(np_list),axis=1,inplace=True)

df_conv=pd.concat([df1_conv,df2_conv],axis=1)
print(df_conv)

#----------------------

df_conv[[glb_list[region],np_list[region]]].iloc[0:10].plot.bar(ax=ax[nax],subplots=False)
ax[nax].set_title('(n) '+figb_title,fontsize=8)
ax[nax].set_xlabel('')
ax[nax].set_ylabel(r'[$\mathrm{TgC}\,\mathrm{yr}^{-1}$]',fontsize=8)
ax[nax].set_xticklabels(dfa.index.values[0:10],rotation=35)
ax[nax].tick_params(labelsize=7)
ax[nax].set_ylim([-110,110])
ax[nax].legend(fontsize=7)

################################
#
nax=14
region=6

# surface buoyancy flux

infl1 = path_glb100km+"ANALY/annclim/hs_pac3_e_sig_sflx_Cant_lyr_anncl.1987-2017.nc"
infl2 = path_np10km+"hst_annclim-np/hs_pac3_e_sig_sflx_Cant_lyr_anncl.1987-2017.nc"
fig_title='Cant diapycnal flux (sflx) summary (1987-2017) for '+item_list[region]
figa_title='diapyc transp. (explicit sflx) '
figb_title='diapyc transp. conv. (explicit sflx) '
sign=1.0

ds_dpy1 = xr.open_dataset(infl1)
ds_dpy1_slim=ds_dpy1.drop_dims(['lat','lon'])
df_dpy1=sign*ds_dpy1_slim.mean(dim='time').to_dataframe()*86400*365.25*1e-12
print(df_dpy1)

ds_dpy2 = xr.open_dataset(infl2)
ds_dpy2_slim=ds_dpy2.drop_dims(['lat','lon'])
df_dpy2=sign*ds_dpy2_slim.mean(dim='time').to_dataframe()*86400*365.25*1e-12
print(df_dpy2)

df_dpy1.set_axis(pd.Index(glb_list),axis=1,inplace=True)
df_dpy2.set_axis(pd.Index(np_list),axis=1,inplace=True)

df_dpy=pd.concat([df_dpy1,df_dpy2],axis=1)
print(df_dpy)

#######

dpy1_flux=df_dpy1.to_numpy()
nz,nx=dpy1_flux.shape
#print(nz,nx)
dpy1_conv=np.array(np.zeros((nz,nx)),dtype=np.float64)

for i in range(nx):
    dpy1_conv[0,i]=dpy1_flux[0,i]

for k in range(1,nz):
    for i in range(nx):
        dpy1_conv[k,i]=dpy1_flux[k,i]-dpy1_flux[k-1,i]

dpy2_flux=df_dpy2.to_numpy()
nz,nx=dpy2_flux.shape
#print(nz,nx)
dpy2_conv=np.array(np.zeros((nz,nx)),dtype=np.float64)

for i in range(nx):
    dpy2_conv[0,i]=dpy2_flux[0,i]

for k in range(1,nz):
    for i in range(nx):
        dpy2_conv[k,i]=dpy2_flux[k,i]-dpy2_flux[k-1,i]

den_half = [21.5,22.5,23.5,24.25,24.75,25.25,25.75,26.25,26.75,27.25]

denidx=pd.Index(data=den_half,name="density")
basidx=pd.Index(data=item_list,name="region")

df1_conv=pd.DataFrame(data=dpy1_conv,index=denidx,columns=basidx)
df2_conv=pd.DataFrame(data=dpy2_conv,index=denidx,columns=basidx)

df1_conv.set_axis(pd.Index(glb_list),axis=1,inplace=True)
df2_conv.set_axis(pd.Index(np_list),axis=1,inplace=True)

df_conv=pd.concat([df1_conv,df2_conv],axis=1)
print(df_conv)

#----------------------

df_conv[[glb_list[region],np_list[region]]].iloc[0:10].plot.bar(ax=ax[nax],subplots=False)
ax[nax].set_title('(o) '+figb_title,fontsize=8)
ax[nax].set_xlabel('')
ax[nax].set_ylabel(r'[$\mathrm{TgC}\,\mathrm{yr}^{-1}$]',fontsize=8)
ax[nax].set_xticklabels(dfa.index.values[0:10],rotation=35)
ax[nax].tick_params(labelsize=7)
ax[nax].set_ylim([-110,110])
ax[nax].legend(fontsize=7)

################################
#
nax=15
region=6

# isopycnal diffusion

infl1 = path_glb100km+"ANALY/annclim/hs_pac3_e_sig_iso_Cant_lyr_anncl.1987-2017.nc"
infl2 = path_np10km+"hst_annclim-np/hs_pac3_e_sig_iso_Cant_lyr_anncl.1987-2017.nc"
fig_title='Cant diapycnal flux (iso) summary (1987-2017) for '+item_list[region]
figa_title='diapyc transp. (explicit iso) '
figb_title='diapyc transp. conv. (explicit iso) '
sign=1.0
ds_dpy1 = xr.open_dataset(infl1)
ds_dpy1_slim=ds_dpy1.drop_dims(['lat','lon'])
df_dpy1=sign*ds_dpy1_slim.mean(dim='time').to_dataframe()*86400*365.25*1e-12
print(df_dpy1)

ds_dpy2 = xr.open_dataset(infl2)
ds_dpy2_slim=ds_dpy2.drop_dims(['lat','lon'])
df_dpy2=sign*ds_dpy2_slim.mean(dim='time').to_dataframe()*86400*365.25*1e-12
print(df_dpy2)

df_dpy1.set_axis(pd.Index(glb_list),axis=1,inplace=True)
df_dpy2.set_axis(pd.Index(np_list),axis=1,inplace=True)

df_dpy=pd.concat([df_dpy1,df_dpy2],axis=1)
print(df_dpy)

#######

dpy1_flux=df_dpy1.to_numpy()
nz,nx=dpy1_flux.shape
#print(nz,nx)
dpy1_conv=np.array(np.zeros((nz,nx)),dtype=np.float64)

for i in range(nx):
    dpy1_conv[0,i]=dpy1_flux[0,i]

for k in range(1,nz):
    for i in range(nx):
        dpy1_conv[k,i]=dpy1_flux[k,i]-dpy1_flux[k-1,i]

dpy2_flux=df_dpy2.to_numpy()
nz,nx=dpy2_flux.shape
#print(nz,nx)
dpy2_conv=np.array(np.zeros((nz,nx)),dtype=np.float64)

for i in range(nx):
    dpy2_conv[0,i]=dpy2_flux[0,i]

for k in range(1,nz):
    for i in range(nx):
        dpy2_conv[k,i]=dpy2_flux[k,i]-dpy2_flux[k-1,i]

den_half = [21.5,22.5,23.5,24.25,24.75,25.25,25.75,26.25,26.75,27.25]

denidx=pd.Index(data=den_half,name="density")
basidx=pd.Index(data=item_list,name="region")

df1_conv=pd.DataFrame(data=dpy1_conv,index=denidx,columns=basidx)
df2_conv=pd.DataFrame(data=dpy2_conv,index=denidx,columns=basidx)

df1_conv.set_axis(pd.Index(glb_list),axis=1,inplace=True)
df2_conv.set_axis(pd.Index(np_list),axis=1,inplace=True)

df_conv=pd.concat([df1_conv,df2_conv],axis=1)
print(df_conv)

#----------------------

df_conv[[glb_list[region],np_list[region]]].iloc[0:10].plot.bar(ax=ax[nax],subplots=False)
ax[nax].set_title('(p) '+figb_title,fontsize=8)
ax[nax].set_xlabel('')
ax[nax].set_ylabel(r'[$\mathrm{TgC}\,\mathrm{yr}^{-1}$]',fontsize=8)
ax[nax].set_xticklabels(dfa.index.values[0:10],rotation=35)
ax[nax].tick_params(labelsize=7)
ax[nax].set_ylim([-20,20])
ax[nax].legend(fontsize=7)


#plt.subplots_adjust(top=0.90,hspace=0.33)
plt.savefig('fig/Cant_storage_rate_all_summary_pac3_E-Subtropical.png')
plt.savefig('fig/Cant_storage_rate_all_summary_pac3_E-Subtropical.pdf')
plt.show()
