# -*- coding: utf-8 -*-
import sys
import json
import numpy as np
import xarray as xr
import matplotlib.pyplot as plt
import matplotlib as mpl
import netCDF4
import datetime

if (len(sys.argv) < 4):
    print ('Usage: ' + sys.argv[0] + ' start_year end_year exp1 exp2')
    sys.exit()

ystr=int(sys.argv[1])
yend=int(sys.argv[2])
nyr=yend-ystr+1

exp1=sys.argv[3]
exp2=sys.argv[4]


suptitle = ' Meridional Overturning Circulation Indo-Pacific (ave. from ' + str(ystr) + ' - ' + str(yend) + ')'
title = [ 'Southern Ocean', 'Atlantic Ocean', 'Indo-Pacific Ocean' ]
xlim  = [ [-90, -28], [-30, 90], [-30, 70] ]
xtlim  = [ [-90, -28], [-20, 90], [-20, 70] ]
xsep   = [ 30, 20, 20 ]

title2 = [ '(a) '+ exp1, '(b) '+ exp2, '(c) ' + exp2 + '-' + exp1 + ' (full depth)', '(d) ' + exp2 + '-' + exp1 + ' (0 - 500 m)' ]

latwoa = np.linspace(-89.5,89.5,num=180)

nyr = yend - ystr + 1
time0 = np.empty(nyr,dtype='object')
for yr in range(ystr,yend+1):
    time0[yr-ystr] = datetime.datetime(yr,1,1)

####################################################################
path_glb100km='../data/glb100km/'
path_np10km='../data/np10km/'
####################################################################
# GLB100km

print( "Loading GLB100km data" )

path_g100 = path_glb100km + 'run-20211104o2/ANALY/annual'
basename_g100 = 'hs_moc'

suffix = '.' + '{:04}'.format(ystr)
moc_file = path_g100 + '/' + basename_g100 + suffix
ncg100 = netCDF4.Dataset(moc_file,'r')
ny = len(ncg100.dimensions['lat'])
nz = len(ncg100.dimensions['depth'])
lat = ncg100.variables['lat'][:]
depth = ncg100.variables['depth'][:]

moc_g100 = np.array(np.empty((nyr,4,nz,ny)),dtype=np.float64)
moc_mod = np.array(np.empty((nz,ny)),dtype=np.float64)

for yr in range(ystr,yend+1):
    suffix = '.' + '{:04}'.format(yr)
    moc_file = path_g100 + '/' + basename_g100 + suffix
    ncg100 = netCDF4.Dataset(moc_file,'r')
    moc_mod[:,:] = ncg100.variables['glb'][0,:,:]
    moc_g100[yr-ystr,0,:,:] = moc_mod[:,:].copy()
    moc_mod[:,:] = ncg100.variables['atl'][0,:,:]
    moc_g100[yr-ystr,1,:,:] = moc_mod[:,:].copy()
    moc_mod[:,:] = ncg100.variables['inp'][0,:,:]
    moc_g100[yr-ystr,2,:,:] = moc_mod[:,:].copy()
    ncg100.close()

moc_g100 = np.where(moc_g100 < -9.0e33, np.NaN, moc_g100)

DS_g100_ann = xr.Dataset({'GLB100km': (['time','basin','depth','lat'], moc_g100),},
                         coords = {'time': time0, 'depth': depth, 'lat': lat } )

####################################################################
# NP10km

print( "Loading NP10km data" )

path_gl = path_np10km + 'hst_ann-glb'
path_np = path_np10km + 'hst_ann-np'
moc_fill = np.array(np.empty((nyr,3,nz,ny)),dtype=np.float64)
basename_gl = 'hs_moc'
basename_np = 'hs_moc'

moc_fill = np.array(np.empty((nyr,4,nz,ny)),dtype=np.float64)
moc_np = np.array(np.empty((nz,ny)),dtype=np.float64)
moc_gl = np.array(np.empty((nz,ny)),dtype=np.float64)

for yr in range(ystr,yend+1):
    suffix = '.' + '{:04}'.format(yr)
    gl_file = path_gl + '/y' + str(yr) + '/' + basename_gl + suffix
    np_file = path_np + '/y' + str(yr) + '/' + basename_np + suffix
    ncgl = netCDF4.Dataset(gl_file,'r')
    ncnp = netCDF4.Dataset(np_file,'r')
    moc_gl[:,:] = ncgl.variables['glb'][0,:,:]
    moc_np[:,:] = ncnp.variables['glb'][0,:,:]
    moc_fill[yr-ystr,0,:,:] = moc_gl[:,:]+moc_np[:,:]
    moc_gl[:,:] = ncgl.variables['atl'][0,:,:]
    moc_np[:,:] = ncnp.variables['atl'][0,:,:]
    moc_fill[yr-ystr,1,:,:] = moc_gl[:,:]+moc_np[:,:]
    moc_gl[:,:] = ncgl.variables['inp'][0,:,:]
    moc_np[:,:] = ncnp.variables['inp'][0,:,:]
    moc_fill[yr-ystr,2,:,:] = moc_gl[:,:]+moc_np[:,:]

moc_fill = np.where(moc_fill < -9.0e33, np.NaN, moc_fill)

DS_fill_ann = xr.Dataset({'NP10km': (['time','basin','depth','lat'], moc_fill),},
                         coords = {'time': time0, 'depth': depth, 'lat': lat } )

#######################################################

da0 = DS_g100_ann['GLB100km'].mean(dim='time').values
da1 = DS_fill_ann['NP10km'].mean(dim='time').values


DS = xr.Dataset({'GLB100km': (['basin','depth','lat'], da0*1e-9),
                 'NP10km': (['basin','depth','lat'], da1*1e-9),
                 'exp2-1': (['basin','depth','lat'], (da1-da0)*1e-9),},
                coords = {'depth': depth, 'lat': lat } )

########################################

fig = plt.figure(figsize=(11,8))
fig.suptitle( suptitle, fontsize=18 )

axes0 = np.array( [ [0.07, 0.6, 0.07, 0.3],
                    [0.15, 0.6, 0.17, 0.3],
                    [0.33, 0.6, 0.15, 0.3],
                    [0.07, 0.53, 0.40, 0.015], ])
ax = [ plt.axes([0.07,0.58,0.4,0.33]),
       plt.axes([0.57,0.58,0.4,0.33]),
       plt.axes([0.07,0.1,0.4,0.33]),
       plt.axes([0.57,0.1,0.4,0.33]), ]

ax_cbar = [ plt.axes(axes0[3]),
            plt.axes(axes0[3]+np.array([0.5,0,0,0])),
            plt.axes(axes0[3]+np.array([0,-0.48,0,0])),
            plt.axes(axes0[3]+np.array([0.5,-0.48,0,0])), ]

bounds1 = np.linspace(-30,30,31)
tbounds1 = np.linspace(-30,30,16)
bounds2 = [-20,-10,-8,-6,-5,-4,-3,-2,-1,0,1,2,3,4,5,6,8,10,20]

cmap = [ 'RdBu_r', 'RdBu_r', 'RdBu_r', 'RdBu_r' ]

item = [ exp1, exp2, 'exp2-1', 'exp2-1' ]

for panel in range(4):
    if item[panel] == exp1 or item[panel] == exp2:
        bounds = bounds1
        ticks_bounds = tbounds1
    elif item[panel] == 'exp2-1':
        bounds = bounds2
        ticks_bounds = bounds2

    if (panel == 3):
        da = DS[item[panel]].sel(depth=slice(0,510))
    else:
        da = DS[item[panel]]

    if (panel < 2):
        m = 2
        im = da.isel(basin=m).plot(ax=ax[panel],cmap=cmap[panel],
                                   levels=bounds,
                                   extend='both',
                                   add_colorbar=False)
        cont = da.isel(basin=m).plot.contour(ax=ax[panel],
                                      colors='black',
                                      levels=bounds,
                                      add_labels=True,
                                      linewidths=1.0)
        #ax[panel].set_title(title[m])
        ax[panel].invert_yaxis()
        ax[panel].set_xlim(xlim[m][0],xlim[m][1])
        ax[panel].set_xticks(np.arange(xlim[m][0],xlim[m][1]+0.1,10))
        ax[panel].set_facecolor('lightgray')

        cbar = plt.colorbar(im,orientation='horizontal',spacing='uniform',
                            ticks=ticks_bounds, cax=ax_cbar[panel])
        cbar.set_label(label='Sv',fontsize=8,x=0.95)
        cbar.ax.tick_params(labelsize=8)

        clevels=cont.levels
        cont.clabel(fmt='%4d',fontsize=9,colors='black',inline_spacing=1)

    else:
        m = 2
        #mp = 0
        im = da.isel(basin=m).plot(ax=ax[panel],cmap=cmap[panel],
                                   levels=bounds,
                                   extend='both',
                                   add_colorbar=False)
        da.isel(basin=m).plot.contour(ax=ax[panel],
                                      colors='black',
                                      levels=bounds,
                                      add_labels=False,
                                      linewidths=1.0)
        #ax[panel].set_title(title[m])
        ax[panel].invert_yaxis()
        ax[panel].set_xlim(xlim[m][0],xlim[m][1])
        ax[panel].set_xticks(np.arange(xlim[m][0],xlim[m][1]+0.1,10))
        ax[panel].set_facecolor('lightgray')
        ax[panel].tick_params(axis='y')
        
        cbar = plt.colorbar(im,orientation='horizontal',spacing='uniform',
                            ticks=ticks_bounds, cax=ax_cbar[panel])
        cbar.set_label(label='Sv',fontsize=8,x=0.95)
        cbar.ax.tick_params(labelsize=8)

fig.text(0.25,0.92,title2[0],fontsize=14,
         horizontalalignment='center',verticalalignment='center')
fig.text(0.75,0.92,title2[1],fontsize=14,
         horizontalalignment='center',verticalalignment='center')
fig.text(0.25,0.45,title2[2],fontsize=14,
         horizontalalignment='center',verticalalignment='center')
fig.text(0.75,0.45,title2[3],fontsize=14,
         horizontalalignment='center',verticalalignment='center')

outpng = './fig/MOC_INP_comp.png'
outpdf = './fig/MOC_INP_comp.pdf'
plt.savefig(outpng, bbox_inches='tight', pad_inches=0.1)
plt.savefig(outpdf, bbox_inches='tight', pad_inches=0.1)
plt.show()
