# -*- coding: utf-8 -*-
import sys
import json
import math
import calendar
import datetime
import numpy as np
from scipy import interpolate
import pandas as pd
import xarray as xr
import matplotlib.pyplot as plt
import matplotlib.ticker as ticker
from matplotlib.dates import drange, date2num, DateFormatter
import netCDF4
from netCDF4 import Dataset, num2date

##########################################

if (len(sys.argv) < 4):
    print ('Usage: ' + sys.argv[0] + ' start_year end_year expname')
    sys.exit()

styr = int(sys.argv[1])
edyr = int(sys.argv[2])
expn = sys.argv[3]

suptitle = expn + ' Volume transport for year ' + str(styr) + ' - ' + str(edyr)

##########################################
# In principle, following should be stored in json file.

nsection=15
title_list = [ "(a) Tsushima",
               "(b) Tsugaru", 
               "(c) Soya", 
               "(d) Tatar",
               "(e) Kuril",
               "(f) Bering", 
               "(g) ITF",
               "(h) Lat12S",
               "(i) Lat12Nw",
               "(j) Lat12Ne",
               "(k) Lat28Nw",
               "(l) Lat42Nw",
               "(m) Lat42Ne",
               "(n) Lon170Wn",
               "(o) Lon170Ws" ]

item_list = [ "Tsushima", "Tsugaru", "Soya", "Tatar", "Kuril", "Bering",
              "ITF", "Lat12S", "Lat12Nw", "Lat12Ne", "Lat28Nw", 
              "Lat42Nw", "Lat42Ne", "Lon170Wn", "Lon170Ws" ]

sign_list = [ -1.0,  1.0, -1.0, -1.0,  1.0, -1.0, 
              -1.0,  1.0, -1.0, -1.0, -1.0,
              -1.0,  1.0, -1.0, -1.0 ]

range_list = [ [-0.1, 1.0], [-0.1, 1.0], [-0.1, 0.5], [-0.1, 0.5], [-0.5, 1.0], [-0.2, 4.0],
               [-2.0, 5.0], [-10.0, 12.0], [-6.0, 12.0], [-6.0, 12.0], [-6.0, 12.0],
               [-5.5, 5.5], [-5.5, 5.5], [-10.0, 10.0], [-10.0, 10.0] ]

##########################################
path_glb100km='../data/glb100km/'
path_np10km='../data/np10km/'
##########################################

num_pen=73

time_pen = np.empty(num_pen,dtype='object')
weight_pen = np.array(np.zeros((num_pen)),dtype=np.float64)
time_mon = np.empty(12,dtype='object')

for year in range(styr,edyr+1):

    leap = calendar.isleap(year)
    print(year,leap)

    time_pen[0] = datetime.datetime(year,1,5)
    for pen in range(1,num_pen):
        if (pen == 11):
            time_pen[pen] = datetime.datetime(year,3,1)
        else:
            time_pen[pen] = time_pen[pen-1] + datetime.timedelta(days=5)

    #print(time_pen)

    weight_pen[:] = 5.0
    if (leap):
        weight_pen[11] = 6.0

    for mon in range(12):
        time_mon[mon] = datetime.datetime(year,mon+1,15)

    weight_year = xr.DataArray(weight_pen,dims=['time'],coords=dict(time=time_pen))

    #####
    print( "Loading pentad data" )

    path_pen=path_np10km+'hst_5day-np'

    base_vl = 'hs_pac3_pass_Volume_lyr.*'
    base_gm = 'hs_pac3_pass_Volume_gm_lyr.*'

    vl_file = path_pen + '/y' + '{:04}'.format(year) + '/' + base_vl
    gm_file = path_pen + '/y' + '{:04}'.format(year) + '/' + base_gm

    DS_vl = xr.open_mfdataset(paths=vl_file,concat_dim='time',combine='nested',decode_times=False)
    DS_vl = DS_vl.assign_coords({'time':time_pen})
    DS_vl = DS_vl*1e-12 # [cm3 s-1] -> [Sv]

    DS_gm = xr.open_mfdataset(paths=gm_file,concat_dim='time',combine='nested',decode_times=False)
    DS_gm = DS_gm.assign_coords({'time':time_pen})
    DS_gm = DS_gm*1e-12 # [cm3 s-1] -> [Sv]

    if (year == styr):
        DS_vl_all = DS_vl.copy()
        DS_gm_all = DS_gm.copy()
        weight_all = weight_year.copy()
    else:
        DS_vl_all = xr.concat([DS_vl_all,DS_vl],dim='time')
        DS_gm_all = xr.concat([DS_gm_all,DS_gm],dim='time')
        weight_all = xr.concat([weight_all,weight_year],dim='time')

    #print(weight_all)
    
print(DS_vl_all)
print(DS_gm_all)
print(weight_all)

DS_vl_all_weighted=DS_vl_all.weighted(weight_all)
DS_vl_all_weighted_mean=DS_vl_all_weighted.mean('time')

DS_gm_all_weighted=DS_gm_all.weighted(weight_all)
DS_gm_all_weighted_mean=DS_gm_all_weighted.mean('time')

#############################################
# output mean to CSV file 

dens=DS_vl_all.coords["depth"].values

df_vl = DS_vl_all_weighted_mean.to_dataframe()
csv_fname='csv/Pac3_Volume_trans_lyr_resolved_'+expn+'_'+str(styr)+'-'+str(edyr)+'.csv'
denidx=pd.Index(data=dens,name="density")
df_vl.set_index(denidx,inplace=True)
print(df_vl)
df_vl.to_csv(csv_fname)

df_gm = DS_gm_all_weighted_mean.to_dataframe()
csv_fname='csv/Pac3_Volume_trans_lyr_gm_'+expn+'_'+str(styr)+'-'+str(edyr)+'.csv'
denidx=pd.Index(data=dens,name="density")
df_gm.set_index(denidx,inplace=True)
print(df_gm)
df_gm.to_csv(csv_fname)

#---

#print(sign_list[0]*df_vl[item_list[0]].values)
#print(sign_list[0]*df_vl[item_list[0]].sum('density').values)
#print(sign_list[0]*DS_vl_all[item_list[0]].mean('time').sum('depth').values)

#print(sign_list[0]*df_gm[item_list[0]].values)
#print(sign_list[0]*df_gm[item_list[0]].sum('density').values)

#print(sign_list[1]*df_vl[item_list[1]].sum('density').values)
#print(sign_list[1]*DS_vl_all[item_list[1]].mean('time').sum('depth').values)

#print(sign_list[2]*df_vl[item_list[2]].sum('density').values)
#print(sign_list[2]*DS_vl_all[item_list[2]].mean('time').sum('depth').values)


#J 描画
fig = plt.figure(figsize=(18,8))
fig.suptitle( suptitle, fontsize=18 )

ax = [ plt.subplot(2,8,1),
       plt.subplot(2,8,2),
       plt.subplot(2,8,3),
       plt.subplot(2,8,4),
       plt.subplot(2,8,5),
       plt.subplot(2,8,6),
       plt.subplot(2,8,7),
       plt.subplot(2,8,8),
       plt.subplot(2,8,9),
       plt.subplot(2,8,10),
       plt.subplot(2,8,11),
       plt.subplot(2,8,12),
       plt.subplot(2,8,13),
       plt.subplot(2,8,14),
       plt.subplot(2,8,15) ]

#new_xticks = date2num(time_mon)

#bounds1 = [0.0, 0.01, 0.02, 0.05, 0.1, 0.2, 0.3, 0.5, 0.7, 1.0, 1.5, 2.0, 3.0, 4.0]
#tbounds1 = [0.0, 0.01, 0.02, 0.05, 0.1, 0.2, 0.3, 0.5, 0.7, 1.0, 1.5, 2.0, 3.0, 4.0]
#bounds1=np.array(np.arange(0, 5.1, 0.25))
#tbounds1=np.array(np.arange(0.0, 5.1, 0.5))
#bounds2=np.array(np.arange(0.0, 2.2, 0.1))
#tbounds2=np.array(np.arange(0.0, 2.2, 0.5))

########

for item in range(nsection):

    print(item_list[item])
    #print(range_list[item][0],range_list[item][1])

    nax=item

    #df_vl=DS_vl_all_weighted_mean[item_list[item]].to_dataframe(name='resolved')
    df_vl_sel = df_vl[item_list[item]].rename('resolved') * sign_list[item]
    #print(df_vl)

    #df_gm=DS_gm_all_weighted_mean[item_list[item]].to_dataframe(name='gm')
    df_gm_sel = df_gm[item_list[item]].rename('gm') * sign_list[item]

    df = pd.concat([df_vl_sel,df_gm_sel],axis=1)
    print(df)

    df_sum = df.sum(axis=0)
    print(df_sum)
    total=df_sum.sum(axis=0)
    print(total)

    df.plot.bar(stacked=True,ax=ax[nax])
    ax[nax].set_title(title_list[item]+'\n total = '+'{:.1f}'.format(total) + '$\,\mathrm{Sv}$')
    #ax[nax].set_title(title_list[item])
    ax[nax].set_xlabel('')
    ax[nax].set_ylabel('[$\mathrm{Sv}$]',fontsize=12)
    ax[nax].set_ylim(range_list[item][0],range_list[item][1])

#nax=0
#da=DS_vl[item_list[nax]].sum(dim='depth')*sign_list[nax]
#da.plot(ax=ax[nax],color='green',linewidth=2)
#ax[nax].set_title(title_list[nax])
#ax[nax].set_xlim(date2num([time_pen[0],time_pen[num_pen-1]]))
#ax[nax].xaxis.set_major_locator(ticker.FixedLocator(new_xticks))
#ax[nax].xaxis.set_major_formatter(DateFormatter('%m%d'))
#ax[nax].tick_params(axis='x',rotation=0)
#ax[nax].set_xlabel('')
#ax[nax].set_ylabel('[$\mathrm{Sv}$]',fontsize=12)

#nax=nax+1
#da=DS_vl[item_list[nax]].sum(dim='depth')*sign_list[nax]
#da.plot(ax=ax[nax],color='green',linewidth=2)
#ax[nax].set_title(title_list[nax])
#ax[nax].set_xlim(date2num([time_pen[0],time_pen[num_pen-1]]))
#ax[nax].xaxis.set_major_locator(ticker.FixedLocator(new_xticks))
#ax[nax].xaxis.set_major_formatter(DateFormatter('%m%d'))
#ax[nax].tick_params(axis='x',rotation=0)
#ax[nax].set_xlabel('')
#ax[nax].set_ylabel('[$\mathrm{Sv}$]',fontsize=12)

#nax=nax+1
#da=DS_vl[item_list[nax]].sum(dim='depth')*sign_list[nax]
#da.plot(ax=ax[nax],color='green',linewidth=2)
#ax[nax].set_title(title_list[nax])
#ax[nax].set_xlim(date2num([time_pen[0],time_pen[num_pen-1]]))
#ax[nax].xaxis.set_major_locator(ticker.FixedLocator(new_xticks))
#ax[nax].xaxis.set_major_formatter(DateFormatter('%m%d'))
#ax[nax].tick_params(axis='x',rotation=0)
#ax[nax].set_xlabel('')
#ax[nax].set_ylabel('[$\mathrm{Sv}$]',fontsize=12)

#nax=nax+1
#da=DS_vl[item_list[nax]].sum(dim='depth')*sign_list[nax]
#da.plot(ax=ax[nax],color='green',linewidth=2)
#ax[nax].set_title(title_list[nax])
#ax[nax].set_xlim(date2num([time_pen[0],time_pen[num_pen-1]]))
#ax[nax].xaxis.set_major_locator(ticker.FixedLocator(new_xticks))
#ax[nax].xaxis.set_major_formatter(DateFormatter('%m%d'))
#ax[nax].tick_params(axis='x',rotation=0)
#ax[nax].set_xlabel('')
#ax[nax].set_ylabel('[$\mathrm{Sv}$]',fontsize=12)

#nax=nax+1
#da=DS_vl[item_list[nax]].sum(dim='depth')*sign_list[nax]
#da.plot(ax=ax[nax],color='green',linewidth=2)
#ax[nax].set_title(title_list[nax])
#ax[nax].set_xlim(date2num([time_pen[0],time_pen[num_pen-1]]))
#ax[nax].xaxis.set_major_locator(ticker.FixedLocator(new_xticks))
#ax[nax].xaxis.set_major_formatter(DateFormatter('%m%d'))
#ax[nax].tick_params(axis='x',rotation=0)
#ax[nax].set_xlabel('')
#ax[nax].set_ylabel('[$\mathrm{Sv}$]',fontsize=12)

plt.tight_layout(rect=[0,0,1,0.95])
outpng = 'fig/Volume_trans_pac3_'+expn+'_'+str(styr)+'-'+str(edyr)+'.png'
outpdf = 'fig/Volume_trans_pac3_'+expn+'_'+str(styr)+'-'+str(edyr)+'.pdf'
plt.savefig(outpng, bbox_inches='tight', pad_inches=0.1)
plt.savefig(outpdf, bbox_inches='tight', pad_inches=0.1)
plt.show()
